/*
 boardCap.c
 ***********************************************************************
 This program captures a picture from our BoardFax by generating the
 poixel clock and sync pulse from the RaspberryPi while reading the 
 pixel data through another GPIO pin of the raspberryPi.

 pin0  output   pixel clock
 pin1  output   sync pulse (begin of the pixel line)
 pin2  input    pixel data
 pin10 input    stepper driver (pin6 seems not to work)


 CC BY-NC-SA 2014 c-hack.de
 2014-12-20 xsider	enhanced for nCLK and  RS pin 
 2014-12-29 ralf    motor detection and write to file added

 *********************************************************************** 
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <wiringPi.h>
#include "boardCap.h"

#define PHI1       16  	/* Clock */
//#define PHI2	    1	/* Clock inverted */
#define	SH	   15	/* Shift Gate */
#define RS	    5	/* Reset Gate */
#define OS     	   12	/* Signal Output, Pixel */
#define MOTOR1	    6	/* Motorpulse1 */
#define MOTOR2	   10	/* Motorpulse2 */
#define PI_ENABLE   1   /* Switch to Raspberry Mode */
 
// Unbenutzt Eingang #13, Komparator tut nicht
// Unbenutzt Eingang #14, Komparator
// Unbenutzt Eingang #11, Komparator

#define PIXEL_SIZE  2048
#define RAW_X 2500
#define RAW_Y 1700
#define LOWER_BORDER 300

#define TRUE 1
#define FALSE 0

// Global Variables
char picture [RAW_X][RAW_Y];
int lastMotorState = HIGH;
char offline = FALSE;

int main(int argc, char *argv[])
{
  int i;
  int columnIdx;

  printf("Raspberry Pi BoardFaxHack\n");
  printf("cc-by-sa-nc  2014   c-hack.de\n\n");
  
  if (argc < 2) {
	  printf("Usage:  ");
	  printf("boardCap filename [-o]\n");
	  printf("-o    offline (I.e. Capturing starts without waiting for motor pulses)\n\n");
	  exit(0);
  } 
  
  if ((argc == 3) && (strcmp(argv[2], "-o") == 0)) {
	  offline = TRUE;
	  printf("offline (no wait for motor clock)!\n");
  }

  wiringPiSetup();
  for (i=0; i<16; i++) pinMode(i,INPUT);
  pinMode(PHI1, OUTPUT);
//  pinMode(PHI2, OUTPUT);
  pinMode(SH, OUTPUT);
  pinMode(RS, OUTPUT);
  pinMode(OS, INPUT);
  pinMode(PI_ENABLE, OUTPUT);
  
  /* ------ Initialize ------------------*/
  digitalWrite(PHI1, LOW);   // SYNC low
//  digitalWrite(PHI2, LOW);    // Clock off  
  digitalWrite(PI_ENABLE, HIGH);   //Switch to PI Mode
  /* ------------------------------------*/

  printf("Capturing Data (waiting for stepper pulses)...\n");
  columnIdx = 0;
  while((risingEdge() == FALSE) && (offline == FALSE)) {   // Wait for a rising Edge of the stepper driver 
     lineCapture(0);                // Dummy capture in order to keep the clock running
  }
  printf("Motor signal detected...\n");
  while (columnIdx < RAW_X) {
     lineCapture(columnIdx);   // Capture with max speed
     columnIdx++;
//     if (columnIdx % 100 == 0) printf("columnIdx = %i\n",columnIdx);  // Debug only
  }
  writeBMPFile(argv[1]);   // write always to the same file
  printf("Capture finished");
  return 0;
}

// Indicate a rising edge of the stepper driver (non blocking)
int risingEdge() {
   int state, retVal;

   state = digitalRead(MOTOR2);
   if ((state == HIGH) && (lastMotorState == LOW)) retVal = TRUE;
   else retVal = FALSE;

   lastMotorState = state;
   return retVal;
}

// Captures one line including overhead
void lineCapture(int x_index) {
   int i;

   digitalWrite(PHI1,HIGH);
 //  digitalWrite(PHI2,LOW);
   digitalWrite(SH,HIGH);   // One SYNC pulse
   delayMicroseconds(2);
   digitalWrite(SH,LOW);
   digitalWrite(PHI1,LOW);
 //  digitalWrite(PHI2,HIGH);
   delayMicroseconds(2);
   for (i=0; i<RAW_Y; i=i+2) {
      digitalWrite(PHI1, HIGH);
 //     digitalWrite(PHI2, LOW);
      delayMicroseconds(1);
      picture[x_index][i] = (char) digitalRead(OS);
      digitalWrite(RS,HIGH);
      digitalWrite(RS,LOW);
      digitalWrite(PHI1, LOW);
 //     digitalWrite(PHI2, HIGH);
      delayMicroseconds(1);
      picture[x_index][i+1] = (char) digitalRead(OS);
      digitalWrite(RS,HIGH);
      digitalWrite(RS,LOW);
   }
}

void writeBMPFile(char *filename) {
  int i,j,x,y;
  int w;  // Width
  int h;  // Height
  unsigned char color;
  
  w = RAW_X;
  h = RAW_Y-LOWER_BORDER;
  
  FILE *f;
  unsigned char *img = NULL;
  int filesize = 54 + 3*w*h;  
  if( img ) free( img );
  img = (unsigned char *)malloc(3*w*h);    // Eigentlich könnte man auf dieses Array verzichten und direkt aus dem picture-Array lesen (TDDO) 
  memset(img,0,sizeof(img));

  for(i=0; i<w; i++)
  {
    for(j=0; j<h; j++)
    {
      x=i; y=(h-1)-j;
      if (picture[i][j+LOWER_BORDER]>0) color = 0; else color = 255; 
      img[(x+y*w)*3+2] = color;   // Hier kommen die Pixel (255, 0 ,0 bedeutet alles rot)
      img[(x+y*w)*3+1] = color;
      img[(x+y*w)*3+0] = color;
    }
  }

  unsigned char bmpfileheader[14] = {'B','M', 0,0,0,0, 0,0, 0,0, 54,0,0,0};
  unsigned char bmpinfoheader[40] = {40,0,0,0, 0,0,0,0, 0,0,0,0, 1,0, 24,0};   // 24 bit Farbtiefe
  unsigned char bmppad[3] = {0,0,0};

  bmpfileheader[ 2] = (unsigned char)(filesize    );
  bmpfileheader[ 3] = (unsigned char)(filesize>> 8);
  bmpfileheader[ 4] = (unsigned char)(filesize>>16);
  bmpfileheader[ 5] = (unsigned char)(filesize>>24);

  bmpinfoheader[ 4] = (unsigned char)(       w    );
  bmpinfoheader[ 5] = (unsigned char)(       w>> 8);
  bmpinfoheader[ 6] = (unsigned char)(       w>>16);
  bmpinfoheader[ 7] = (unsigned char)(       w>>24);
  bmpinfoheader[ 8] = (unsigned char)(       h    );
  bmpinfoheader[ 9] = (unsigned char)(       h>> 8);
  bmpinfoheader[10] = (unsigned char)(       h>>16);
  bmpinfoheader[11] = (unsigned char)(       h>>24);

  f = fopen(filename,"wb");
  fwrite(bmpfileheader,1,14,f);
  fwrite(bmpinfoheader,1,40,f);
  for(i=0; i<h; i++)
  {
    fwrite(img+(w*(h-i-1)*3),3,w,f);
    fwrite(bmppad,1,(4-(w*3)%4)%4,f);
  }
  fclose(f);
}

